/*
 * adc.c
 *
 *  Created on: Jun 24, 2023
 *      Author: hussamaldean
 */


#include "adc.h"
#include "stm32f4xx.h"                  // Device header
#include "stdio.h"


uint16_t adc_data1[10];
uint16_t adc_data2[10];
volatile uint8_t finished=0;


void adc_init(void)
{
	/*ADC related set up*/
	RCC->AHB1ENR		|=RCC_AHB1ENR_GPIOAEN;
	GPIOA->MODER		|=GPIO_MODER_MODE0_0|GPIO_MODER_MODE0_1;

	RCC->APB2ENR		|=RCC_APB2ENR_ADC1EN;
	ADC1->CR2			|=ADC_CR2_DMA|ADC_CR2_DDS;
	ADC1->CR2			|=ADC_CR2_EXTEN_0;
	ADC1->CR2			|=ADC_CR2_EXTSEL_1|ADC_CR2_EXTSEL_2;

	/*DMA related setup*/
	RCC->AHB1ENR		|=RCC_AHB1ENR_DMA2EN;
	DMA2_Stream0->CR	&=~DMA_SxCR_EN;
	while(DMA2_Stream0->CR ==DMA_SxCR_EN){;}

	DMA2_Stream0->CR	|=DMA_SxCR_MSIZE_0|DMA_SxCR_PSIZE_0|
			DMA_SxCR_MINC|DMA_SxCR_TCIE|DMA_SxCR_HTIE
			|DMA_SxCR_CIRC|DMA_SxCR_DBM;

	DMA2_Stream0->PAR	 =(uint32_t)(&(ADC1->DR));
	DMA2_Stream0->M0AR 	 =(uint32_t )(adc_data1);
	DMA2_Stream0->M1AR 	 =(uint32_t )(adc_data2);
	DMA2_Stream0->NDTR	 =10;

	NVIC_EnableIRQ(DMA2_Stream0_IRQn);

	/* Timer related setup*/
	RCC->APB1ENR		|=RCC_APB1ENR_TIM2EN;
	TIM2->PSC			 =16000-1;
	TIM2->ARR			 =10-1;
	TIM2->CR2			|=TIM_CR2_MMS_1;
	/*Launch the ADC*/
	ADC1->CR2			|=ADC_CR2_ADON;
	/*Launch the DMA*/
	DMA2_Stream0->CR 	|=DMA_SxCR_EN;
	/*Launch the timer*/
	TIM2->CR1			|=TIM_CR1_CEN;
}


uint8_t buffer_filled()
{
	return finished;
}


void print_buffer()
{

	finished=0;

	if ((DMA2_Stream0->CR & DMA_SxCR_CT)) /*If true, adc_data1 is filled */
	{
		printf("ADC Buffer1 data are:\r\n");
		for(int i=0;i<10;i++)
		{
			printf("%d\t",adc_data1[i]);
		}
		printf("\r\n");
	}

	else
	{
		printf("ADC Buffer2 data are:\r\n");
		for(int i=0;i<10;i++)
		{
			printf("%d\t",adc_data2[i]);
		}
		printf("\r\n");

	}

}


void DMA2_Stream0_IRQHandler(void)
{

	if(((DMA2->LISR)&DMA_LISR_TCIF0))
	{
		finished=1;
		DMA2->LIFCR=DMA_LIFCR_CTCIF0;
	}

	if(((DMA2->LISR)&DMA_LISR_HTIF0))
	{
		finished=1;
		DMA2->LIFCR=DMA_LIFCR_CHTIF0;
	}

}
